from graphics_and_games_klassen import Zeichenfenster, Text
from helferklassen import Knoten, KantenSymbol, Labyrinth, Theseus

# Testklasse für die Korrespondez Labyrinth zu Graph.
class Test:

    # Legt die benötigten Objekte an.
    def __init__(self):
        self.l = Labyrinth(40, 26)
        self.l.Generieren()
        self.g = GraphMatrix()
        self.l.GraphErzeugen(self.g)
    
    # Zeigt nur den Graphen an.
    def GraphAnzeigen(self):
        self.l.SichtbarkeitSetzen(False)
        self.g.SichtbarkeitSetzen(True)
    
    # Zeigt nur das Labyrinth an.
    def LabyrinthAnzeigen(self):
        self.l.SichtbarkeitSetzen(True)
        self.g.SichtbarkeitSetzen(False)


# Rahmenklasse für die Wegesuche aus dem Labyrith.
class Rahmen:

    # Legt die benötigten Objekte an.
    def __init__(self):
        self.l = Labyrinth(40, 26)
        self.l.Generieren()
        self.g = GraphMatrix()
        self.l.GraphErzeugen(self.g)
        self.t = None
    
    # Zeigt nur den Graphen an.
    def GraphAnzeigen(self):
        self.l.SichtbarkeitSetzen(False)
        self.g.SichtbarkeitSetzen(True)
        if self.t is not None:
            self.t.SichtbarkeitSetzen(False)
    
    # Zeigt nur das Labyrinth an.
    def LabyrinthAnzeigen(self):
        self.l.SichtbarkeitSetzen(True)
        self.g.SichtbarkeitSetzen(False)
        if self.t is not None:
            self.t.SichtbarkeitSetzen(True)
    
    # Positioniert den verirrten Theseus auf einen geeigenten Startpunkt
    def TheseusPositionieren(self):
        if self.t is not None:
            self.t.Entfernen()
        self.LabyrinthAnzeigen()
        self.l.StartpunktGenerieren()
        self.t = Theseus(self.l.XStartGeben(), self.l.YStartGeben())
    
    # Ermittelt den Weg aus dem Labyrinth und bereitet den Gang zu Ausgang vor.
    def FluchtwegBerechnen(self):
        if self.t is not None:
            self.t.WegSetzen(self.g.BreitensucheAusfuehren(self.l.XStartGeben(), self.l.YStartGeben(), self.l.XAusgangGeben(), self.l.YAusgangGeben()))


class GraphMatrix:

    # Baut die Datenstruktur auf
    def __init__(self):
        self.knoten = []
        self.kanten = []
        self.matrix = []

    # Einfügen eines neuen Knoten in den Graphen
    def KnotenEinfuegen(self, x, y):
        self.knoten.append(Knoten(x, y))
        for zeile in self.matrix:
            zeile.append(-1)
        self.matrix.append([])
        for zeile in self.matrix:
            self.matrix[-1].append(-1)

    # Gibt den Knoten mit dem angegebenen Bezeichner zurück.
    def KnotenGeben(self, x, y):
        for k in self.knoten:
            if k.XGeben() == x and k.YGeben() == y:
                return k
        return None

    # Gibt die interne Nummer des Knotens
    def KnotenNummerGeben(self, x, y):
        for i in range(len(self.knoten)):
            if self.knoten[i].XGeben() == x and self.knoten[i].YGeben() == y:
                return i
        return -1

    # Gibt die interne Nummer des Knotens
    def KnotenNummerGeben2(self, knoten):
        for i in range(len(self.knoten)):
            if self.knoten[i] == knoten:
                return i
        return -1

    # Gibt die Bezeichnung eines Knotens mit der internen Knotennummer
    def KnotenBezeichnerGeben(self, KnotenNummer):
        if knotenNummer < len(self.knoten) and knotenNummer >= 0:
            return self.knoten[knotenNummer].BezeichnerGeben()
        else:
            return ""

    # Einfügen einer Kante in den Graphen
    def KanteEinfuegen(self, x1, y1, x2, y2):
        vonNummer = self.KnotenNummerGeben(x1, y1)
        nachNummer = self.KnotenNummerGeben(x2, y2)
        if vonNummer != -1 and nachNummer != -1 and vonNummer != nachNummer:
            self.matrix[vonNummer][nachNummer] = 1
            self.matrix[nachNummer][vonNummer] = 1
            self.kanten.append(KantenSymbol(self.knoten[vonNummer].SymbolGeben(), self.knoten[nachNummer].SymbolGeben(), False, "", 3, "blau"))

    # Gibt die Anzahl der Knoten des Graphen zurück
    def KnotenAnzahlGeben(self):
        return len(self.knoten)

    # Gibt die Gewichtung einer Kante zurück
    # Die Kante ist durch einen Anfangsknoten und einen Endknoten festgelegt
    def KanteGewichtGeben(self, von, nach):
        vonNummer = self.KnotenNummerGeben(von)
        nachNummer = self.KnotenNummerGeben(nach)
        if vonNummer != -1 and nachNummer!=-1:
            return self.matrix[vonNummer][nachNummer]
        else:
            return -1
    
    # Setzt die Sichtbarkeit des Graphen
    def SichtbarkeitSetzen(self, sichtbar):
        for k in self.kanten:
            k.SichtbarkeitSetzen(sichtbar)
        for k in self.knoten:
            k.SymbolGeben().SichtbarkeitSetzen(sichtbar)

    # Löscht die Kanten und Knoten des Graphen
    # Die Anzeige wird auch gelöscht
    def ZurueckSetzen(self):
        for k in self.kanten:
            k.Entfernen()
        for k in self.knoten:
            k.SymbolGeben().Entfernen()
        self.knoten.clear()
        self.kanten.clear()
        self.matrix.clear()
    
    # Führt die Breitensuche mit dem angegebenen Start- und Zielknoten durch
    # Falls der Zielknoten erreicht wurde, wird am Ende der Weg dorthin zurück gegeben, sonst ein leerer Weg
    def BreitensucheAusfuehren (self, x1, y1, x2, y2):
        fertig = []
        inArbeit = []
        weg = []
        # Breitensuchealgorithmus ergänzen
        return weg
