from graphics_and_games_klassen import Kreis, Text, Rechteck, Dreieck, Zeichenfenster, Figur
import math, sqlite3, random


# Theseus erhält einen Weg, auf dem er zum Ausgang findet
class Theseus(Figur):
    
    # Legt die Figur und die Startposition fest.
    def __init__(self, x, y):
        super().__init__()
        self.weg = None
        self.schritte = 0
        self.xPos = x
        self.yPos = y
        self.GroesseSetzen(LabyrinthElement.seitenlaenge)
        self.FigurFestlegen()
        self.PositionSetzen(int(x * LabyrinthElement.seitenlaenge + LabyrinthElement.seitenlaenge / 2), int(y  * LabyrinthElement.seitenlaenge + LabyrinthElement.seitenlaenge / 2))
    
    # Legt das Aussehen von Theseus fest.
    def FigurFestlegen(self):
        self.EigeneFigurLoeschen()
        self.FigurteilFestlegenRechteck(-35, 0, 70, 25, "rot")
        self.FigurteilFestlegenEllipse(-25, 25, 10, 10, "schwarz")
        self.FigurteilFestlegenEllipse(15, 25, 10, 10, "schwarz")
        self.FigurteilFestlegenEllipse(-20, -40, 40, 40, "rot")
        self.FigurteilFestlegenEllipse(5, -27, 15, 15, "gelb")
    
    # Legt den Weg zum Ausgang fest.
    def WegSetzen(self, weg):
        self.weg = weg
        self.schritte = 1
    
    # Bewegt Theseus ein Feld in Richtung Ausgang
    def AktionAusfuehren(self):
        if self.weg is not None and self.schritte < len(self.weg):
            k = self.weg[self.schritte]
            if k.XGeben() == self.xPos + 1:
                self.WinkelSetzen(0)
            elif k.XGeben() == self.xPos - 1:
                self.WinkelSetzen(180)
            elif k.YGeben() == self.yPos - 1:
                self.WinkelSetzen(90)
            elif k.YGeben() == self.yPos + 1:
                self.WinkelSetzen(270)
            self.Gehen(LabyrinthElement.seitenlaenge)
            self.xPos = k.XGeben()
            self.yPos = k.YGeben()
            self.schritte += 1


# Oberklasse aller Bausteine des Labyrinths
class LabyrinthElement:
    
    seitenlaenge = 20

    # Legt das Darstellungsrechteck an.
    def __init__(self, x, y):
        self.x = x
        self.y = y
        self.darstellung = Rechteck()
        self.darstellung.PositionSetzen(x * LabyrinthElement.seitenlaenge, y * LabyrinthElement.seitenlaenge)
        self.darstellung.GroesseSetzen(LabyrinthElement.seitenlaenge, LabyrinthElement.seitenlaenge)
    
    # Gibt die x-Position des Elements zurück.
    def XPositionGeben(self):
        return self.x
    
    # Gibt die y-Position des Elements zurück.
    def YPositionGeben(self):
        return self.y
    
    # Setzt die Sichtbarkeit des Elements
    def SichtbarkeitSetzen(self, sichtbar):
        self.darstellung.SichtbarkeitSetzen(sichtbar)


# Ein Mauerbaustein des Labyrinths
class Mauer(LabyrinthElement):

    # Legt das Darstellungsrechteck in der Mauerfarbe an.
    def __init__(self, x, y):
        super().__init__(x, y)
        self.darstellung.FarbeSetzen("schwarz")


# Ein Gangbaustein des Labyrinths
class Gang(LabyrinthElement):

    # Legt das Darstellungsrechteck in der Gangfarbe an.
    def __init__(self, x, y):
        super().__init__(x, y)
        self.darstellung.FarbeSetzen("weiss")


# Ein Ausgangbaustein des Labyrinths
class Ausgang(Gang):

    # Legt das Darstellungsrechteck in der Ausgangfarbe an.
    def __init__(self, x, y):
        super().__init__(x, y)
        self.darstellung.FarbeSetzen("gruen")


# Verwaltet das Labyrinth
class Labyrinth:

    # Legt die Attribute an.
    def __init__(self, breite, hoehe):
        self.elemente = []
        for b in range(breite):
            self.elemente.append([])
            for h in range(hoehe):
                self.elemente[b].append(None)
        self.xAusgang = -1
        self.yAusgang = -1
        self.xStart = -1
        self.yStart = -1
    
    # Generiert das Labyrinth
    def Generieren(self):
        dx = 0
        dy = 0
        xmax = len(self.elemente)
        ymax = len(self.elemente[0])
        minSchritte = int(xmax * ymax / 3)
        schritte = 1
        x = random.randint(0, xmax - 3) + 1
        y = random.randint(0, ymax - 3) + 1
        self.elemente[x][y] = Gang(x, y)
        
        while True:
            dx = random.randint(0, 2) - 1
            dy = random.randint(0, 2) - 1
            while abs(dx) == abs(dy):
                dx = random.randint(0, 2) - 1
                dy = random.randint(0, 2) - 1
            x += dx
            y += dy
            if x == 0 or y == 0 or x == xmax - 1 or y == ymax - 1:
                if schritte > minSchritte and self.elemente[x][y] is None:
                    self.elemente[x][y] = Ausgang(x, y)
                    self.xAusgang = x
                    self.yAusgang = y
                    break
                else:
                    x -= dx
                    y -= dy
            elif self.elemente[x][y] is None:
                if self.elemente[x - 1][y - 1] is not None and self.elemente[x - 1][y] is not None and self.elemente[x][y - 1] is not None or self.elemente[x + 1][y - 1] is not None and self.elemente[x + 1][y] is not None and self.elemente[x][y - 1] is not None or self.elemente[x + 1][y + 1] is not None and self.elemente[x + 1][y] is not None and self.elemente[x][y + 1] is not None or self.elemente[x - 1][y + 1] is not None and self.elemente[x - 1][y] is not None and self.elemente[x][y + 1] is not None:
                    x -= dx
                    y -= dy
                else:
                    schritte += 1
                    self.elemente[x][y] = Gang(x, y)
        for b in range(len(self.elemente)):
            for h in range(len(self.elemente[b])):
                if self.elemente[b][h] is None:
                    self.elemente[b][h] = Mauer(b, h)

    # Erzeugt die Graphendarstellung
    def GraphErzeugen(self, graph):
        for x in range(len(self.elemente)):
            for y in range(len(self.elemente[x])):
                if not isinstance(self.elemente[x][y], Mauer):
                    graph.KnotenEinfuegen(x, y)
                    if isinstance(self.elemente[x][y], Ausgang):
                        graph.KnotenGeben(x, y).FarbeSetzen("gruen")
                    if x > 0 and not isinstance(self.elemente[x - 1][y], Mauer):
                        graph.KanteEinfuegen(x, y, x - 1, y)
                    if y > 0 and not isinstance(self.elemente[x][y - 1], Mauer):
                        graph.KanteEinfuegen(x, y, x, y - 1)
    
    # Setzt die Sichtbarkeit des Labyrinths
    def SichtbarkeitSetzen(self, sichtbar):
        for x in range(len(self.elemente)):
            for y in range(len(self.elemente[x])):
                self.elemente[x][y].SichtbarkeitSetzen(sichtbar)
    
    # Meldet die x-Koordinate des Ausgangs zurück
    def XAusgangGeben(self):
        return self.xAusgang
    
    # Meldet die y-Koordinate des Ausgangs zurück
    def YAusgangGeben(self):
        return self.yAusgang
    
    # Startpunkt ermitteln
    def StartpunktGenerieren(self):
        x1 = random.randint(0, len(self.elemente) - 3) + 1
        y1 = random.randint(0, len(self.elemente[0]) - 3) + 1
        while isinstance(self.elemente[x1][y1], Mauer):
            x1 = random.randint(0, len(self.elemente) - 3) + 1
            y1 = random.randint(0, len(self.elemente[0]) - 3) + 1
        x2 = random.randint(0, len(self.elemente) - 3) + 1
        y2 = random.randint(0, len(self.elemente[0]) - 3) + 1
        while isinstance(self.elemente[x2][y2], Mauer):
            x2 = random.randint(0, len(self.elemente) - 3) + 1
            y2 = random.randint(0, len(self.elemente[0]) - 3) + 1
        x3 = random.randint(0, len(self.elemente) - 3) + 1
        y3 = random.randint(0, len(self.elemente[0]) - 3) + 1
        while isinstance(self.elemente[x3][y3], Mauer):
            x3 = random.randint(0, len(self.elemente) - 3) + 1
            y3 = random.randint(0, len(self.elemente[0]) - 3) + 1
        if abs(x1 - self.xAusgang) + abs(y1 - self.yAusgang) > abs(x2 - self.xAusgang) + abs(y2 - self.yAusgang):
            if abs(x1 - self.xAusgang) + abs(y1 - self.yAusgang) > abs(x3 - self.xAusgang) + abs(y3 - self.yAusgang):
                self.xStart = x1
                self.yStart = y1
            elif abs(x2 - self.xAusgang) + abs(y2 - self.yAusgang) > abs(x3 - self.xAusgang) + abs(y3 - self.yAusgang):
                self.xStart = x2
                self.yStart = y2
            else:
                self.xStart = x3
                self.yStart = y3
        elif abs(x2 - self.xAusgang) + abs(y2 - self.yAusgang) > abs(x3 - self.xAusgang) + abs(y3 - self.yAusgang):
            self.xStart = x2
            self.yStart = y2
        else:
            self.xStart = x3
            self.yStart = y3
    
    # Meldet die x-Koordinate des Ausgangs zurück
    def XStartGeben(self):
        return self.xStart
    
    # Meldet die y-Koordinate des Ausgangs zurück
    def YStartGeben(self):
        return self.yStart
    

# Verwaltet ein Knotensymbol in einem Graph
class KnotenSymbol():

    # Legt das Symbol an und besetzt die Attribute.
    def __init__(self, x, y, radius, farbe, bezeichner):
        self.x = x
        self.y = y
        self.r = radius
        self.farbe = farbe
        self.bezeichner = bezeichner
        self.aussen = Kreis()
        self.aussen.FarbeSetzen("schwarz")
        self.innen = Kreis()
        self.text = Text()
        self.text.TextGroesseSetzen(18)
        self.DarstellungAktualisieren()

    # Aktualisiert das Symbol
    def DarstellungAktualisieren(self):
        self.aussen.PositionSetzen(self.x, self.y)
        self.aussen.RadiusSetzen(self.r)
        self.innen.PositionSetzen(self.x, self.y)
        self.innen.RadiusSetzen(self.r - 2)
        self.innen.FarbeSetzen(self.farbe)
        self.text.PositionSetzen(int(self.x - len(self.bezeichner) / 2 * 10 - len(self.bezeichner) % 2 * 5), self.y - 6)
        self.text.TextSetzen(self.bezeichner)

    # Setzt die Farbe der Darstellung
    def FarbeSetzen(self, f):
        self.farbe = f
        self.DarstellungAktualisieren()

    # Setzt die Position der Darstellung
    def PositionSetzen(self, x, y):
        self.x = x
        self.y = y
        self.DarstellungAktualisieren()

    # Setzt die Farbe der Darstellung
    def BezeichnerSetzen(self, bezeichner):
        self.bezeichner = bezeichner
        self.DarstellungAktualisieren()

    # Meldet die x-Koordinate des Symbols zurück.
    def XGeben(self):
        return self.x

    # Meldet die y-Koordinate des Symbols zurück.
    def YGeben(self):
        return self.y

    # Meldet den Bezeichner des Symbols zurück.
    def BezeichnerGeben(self):
        return self.bezeichner

    # Entfernt das Knotensymbol aus der Anzeige
    def Entfernen(self):
        self.aussen.Entfernen()
        self.innen.Entfernen()
        self.text.Entfernen()
    
    # Setzt die Sichtbarkeit des Symbols
    def SichtbarkeitSetzen(self, sichtbar):
        self.aussen.SichtbarkeitSetzen(sichtbar)
        self.innen.SichtbarkeitSetzen(sichtbar)
        self.text.SichtbarkeitSetzen(sichtbar)

# Verwaltet ein Kantensymbol in einem Graph
class KantenSymbol():

    # Legt das Symbol an und besetzt die Attribute.
    def __init__(self, start, ende, gerichtet, gewicht, breite, farbe):
        self.von = start
        self.nach = ende
        self.farbe = farbe
        self.breite = breite
        self.gerichtet = gerichtet
        self.gewicht = gewicht
        self.aussen = Rechteck()
        self.innen = Rechteck()
        self.pfeil = Dreieck()
        self.pfeil.FarbeSetzen("schwarz")
        self.text = Text()
        self.text.SichtbarkeitSetzen(gewicht != "")
        self.text.TextSetzen(gewicht)
        self.DarstellungAktualisieren()

    # Aktualisiert das Symbol
    def DarstellungAktualisieren(self):
        x1 = self.von.XGeben()
        y1 = self.von.YGeben()
        x2 = self.nach.XGeben()
        y2 = self.nach.YGeben()
        laenge = int(math.sqrt((x1 - x2) * (x1 - x2) + (y1 - y2) * (y1 - y2)))
        xm = int((x1 + x2) / 2.0)
        ym = int((y1 + y2) / 2.0)
        x = int(xm - laenge / 2)
        y = int(ym - self.breite / 2)
        self.aussen.PositionSetzen(x, y)
        self.aussen.GroesseSetzen(laenge, self.breite)
        if (self.breite < 4):
            groesse = self.breite * 4
            self.innen.SichtbarkeitSetzen(False)
            self.aussen.FarbeSetzen(self.farbe)
            self.pfeil.SichtbarkeitSetzen(self.gerichtet)
            self.pfeil.PositionSetzen(xm, int(ym - groesse / 2))
            self.pfeil.GroesseSetzen(groesse, groesse)
            self.pfeil.WinkelSetzen(270)
        else:
            delta = 2 if self.breite <= 8 else 4
            self.innen.SichtbarkeitSetzen(True)
            self.innen.PositionSetzen(x, int(y + delta / 2))
            self.innen.GroesseSetzen(laenge, self.breite - delta)
            self.innen.FarbeSetzen(self.farbe)
            self.aussen.FarbeSetzen("schwarz")
            self.pfeil.SichtbarkeitSetzen(self.gerichtet)
            self.pfeil.PositionSetzen(xm, int(ym - self.breite / 2))
            self.pfeil.GroesseSetzen(self.breite - delta, self.breite)
            self.pfeil.WinkelSetzen(270)

        winkel = 0.0
        if x1 == x2:
            if y2 > y1:
                winkel = -90
            else:
                winkel = 90
        elif x1 < x2:
            winkel = -math.atan((y2 - y1) / (x2 - x1)) / math.pi * 180.0
        else:
            winkel = 180 - math.atan((y2 - y1) / (x2 - x1)) / math.pi * 180.0
        self.pfeil.WinkelSetzen(int(winkel + 270))
        self.innen.WinkelSetzen(int(winkel))
        self.aussen.WinkelSetzen(int(winkel))
        self.pfeil.GanzNachHintenBringen()
        self.innen.GanzNachHintenBringen()
        self.aussen.GanzNachHintenBringen()
        if ((-90 <= winkel) and (winkel <= 0)):
            w = (winkel + 90) / 180.0 * math.pi
            delta = self.breite
            self.text.PositionSetzen(xm + int(delta * math.cos(w)), ym - int(delta * math.sin(w)))
        elif ((0 < winkel) and (winkel <= 90)):
            w = (winkel + 90) / 180.0 * math.pi
            delta = self.breite
            self.text.PositionSetzen(xm - int(delta * math.cos(w)), ym + int(delta * math.sin(w)) + 7)
        elif ((90 < winkel) and (winkel <= 180)):
            w = (winkel + 90) / 180.0 * math.pi
            delta = self.breite
            self.text.PositionSetzen(xm - int(delta * math.cos(w)), ym + int(delta * math.sin(w)))
        else:
            w = (winkel + 90) / 180.0 * math.pi
            delta = self.breite
            self.text.PositionSetzen(xm + int(delta * math.cos(w)), ym - int(delta * math.sin(w)) + 10)
    
    # Entfernt das Kantensymbol aus der Anzeige
    def Entfernen(self):
        self.aussen.Entfernen()
        self.innen.Entfernen()
        self.pfeil.Entfernen()
        self.text.Entfernen()
    
    # Setzt die Sichtbarkeit des Symbols
    def SichtbarkeitSetzen(self, sichtbar):
        self.aussen.SichtbarkeitSetzen(sichtbar)
        self.innen.SichtbarkeitSetzen(sichtbar and self.breite >= 4)
        self.text.SichtbarkeitSetzen(sichtbar)
        self.pfeil.SichtbarkeitSetzen(sichtbar and self.gerichtet)

    # Setzt die Farbe der Darstellung
    def FarbeSetzen(self, f):
        self.farbe = f
        self.DarstellungAktualisieren()

    # Meldet den Startknoten
    def StartsymbolGeben(self):
        return self.von

    # Meldet den Zielknoten
    def ZielsymbolGeben(self):
        return self.nach


# Beschreibt einen Knoten
class Knoten:

    # Besetzt die Attribute und legt das Knotensymbol an.
    def __init__(self, x, y):
        self.x = x
        self.y = y
        self.symbol = KnotenSymbol(int(x * LabyrinthElement.seitenlaenge + LabyrinthElement.seitenlaenge / 2), int(y * LabyrinthElement.seitenlaenge + LabyrinthElement.seitenlaenge / 2), (LabyrinthElement.seitenlaenge - 4) / 2, "weiss", "")
        self.vorgaenger = None

    # Meldet das Symbol des Knotens zurück.
    def SymbolGeben(self):
        return self.symbol

    # Setzt die Farbe der Darstellung
    def FarbeSetzen(self, f):
        self.symbol.FarbeSetzen(f)
    
    # Meldet die x-Koordinate des Knotens zurück.
    def XGeben(self):
        return self.x
    
    # Meldet die y-Koordinate des Knotens zurück.
    def YGeben(self):
        return self.y
    
    # Meldet den Vorgänger des Knotens zurück.
    def VorgaengerGeben(self):
        return self.vorgaenger
    
    # Setzt den Vorgänger für den Weg zum Ausgang
    def VorgaengerSetzen(self, v):
        self.vorgaenger = v
