from graphics_and_games_klassen import Zeichenfenster, Text
from helferklassen import Knoten, KantenSymbol, Lesen


# Löst das klassische Problem der Flussüberquerung durch Bauer, Wolf, Ziege und Kohl
class GraphBauer:
    
    # Legt das Graphenobjekt an.
    def __init__(self):
        self.g = GraphMatrix()
        l = Lesen()
        if l.LesenDatenbank('GraphBauer.grdb', self.g):
            self.g.Ausgeben()
            self.g.KnotenGeben('BWKZ|-').ZielknotenSetzen()
        
    # Führt die Breitensuche zur Ermittlung der Überquerungsstrategie aus
    def BreitensucheAusfuehren(self):
        self.g.BreitensucheAusfuehren()


# Löst die Studiofahrt der Gesangscoaches
class GraphCoaches:

    # Legt das Graphenobjekt an.
    def __init__(self):
        self.g = GraphMatrix()
        l = Lesen()
        # Für Teilaufgabe c entsprechend ergänzen, Graphendatenbank: "GraphCoaches.grdb", Zielknotenbezeichner "C1S1C2S2C3S3|-|-"
        
    # Führt die Breitensuche zur Ermittlung der Überquerungsstrategie aus
    def BreitensucheAusfuehren(self):
        self.g.BreitensucheAusfuehren()


class GraphMatrix:

    # Baut die Datenstruktur auf
    def __init__(self):
        self.knoten = []
        self.kanten = []
        self.matrix = []
        self.warteliste = []
        self.fertigeKnoten = []

    # Einfügen eines neuen Knoten in den Graphen
    def KnotenEinfuegen(self, bezeichner, x, y):
        self.knoten.append(Knoten(bezeichner, x, y))
        for zeile in self.matrix:
            zeile.append(-1)
        self.matrix.append([])
        for zeile in self.matrix:
            self.matrix[-1].append(-1)

    # Gibt den Knoten mit dem angegebenen Bezeichner zurück.
    def KnotenGeben(self, bezeichner):
        for k in self.knoten:
            if k.BezeichnerGeben() == bezeichner:
                return k
        return None

    # Gibt die interne Nummer des Knotens
    def KnotenNummerGeben(self, bezeichner):
        for i in range(len(self.knoten)):
            if self.knoten[i].BezeichnerGeben() == bezeichner:
                return i
        return -1

    # Gibt die Bezeichnung eines Knotens mit der internen Knotennummer
    def KnotenBezeichnerGeben(self, KnotenNummer):
        if knotenNummer < len(self.knoten) and knotenNummer >= 0:
            return self.knoten[knotenNummer].BezeichnerGeben()
        else:
            return ""

    # Einfügen einer Kante in den Graphen
    def KanteEinfuegen(self, von, nach, gewichtung):
        vonNummer = self.KnotenNummerGeben(von)
        nachNummer = self.KnotenNummerGeben(nach)
        if vonNummer != -1 and nachNummer != -1 and vonNummer != nachNummer:
            self.matrix[vonNummer][nachNummer] = 1
            self.matrix[nachNummer][vonNummer] = 1
            self.kanten.append(KantenSymbol(self.knoten[vonNummer].SymbolGeben(), self.knoten[nachNummer].SymbolGeben(), False, str(gewichtung), 3, "blau"))

    # Gibt die Adjazenzmatrix des Graphen in der Konsole aus
    def Ausgeben(self):
        # Kopfzeile
        print("    ", end='')
        for k in self.knoten:
            print((k.BezeichnerGeben() + "    ")[0:4], end='')
        print("")

        for i in range(len(self.knoten)):
            print((self.knoten[i].BezeichnerGeben() + "    ")[0:4], end='')
            for j in range(len(self.knoten)):
                if self.matrix[i][j] != -1:
                    print((str(self.matrix[i][j]) + "   ")[0:4], end='')
                else:
                    print("    ", end='')
            print("")

    # Gibt die Anzahl der Knoten des Graphen zurück
    def KnotenAnzahlGeben(self):
        return len(self.knoten)

    # Gibt die Gewichtung einer Kante zurück
    # Die Kante ist durch einen Anfangsknoten und einen Endknoten festgelegt
    def KanteGewichtGeben(self, von, nach):
        vonNummer = self.KnotenNummerGeben(von)
        nachNummer = self.KnotenNummerGeben(nach)
        if vonNummer != -1 and nachNummer!=-1:
            return self.matrix[vonNummer][nachNummer]
        else:
            return -1
    
    # Setzt die Sichtbarkeit des Graphen
    def SichtbarkeitSetzen(self, sichtbar):
        for k in self.kanten:
            k.SichtbarkeitSetzen(sichtbar)
        for k in self.knoten:
            k.SymbolGeben().SichtbarkeitSetzen(sichtbar)

    # Löscht die Kanten und Knoten des Graphen
    # Die Anzeige wird auch gelöscht
    def ZurueckSetzen(self):
        for k in self.kanten:
            k.Entfernen()
        for k in self.knoten:
            k.SymbolGeben().Entfernen()
        self.knoten.clear()
        self.kanten.clear()
        self.matrix.clear()
    
    # Führt die Breitensuche mit dem angegebenen Start- und Zielknoten durch
    def BreitensucheAusfuehren (self):
        # Gemäß den Vorgaben der Aufgabe und dem allgemeinen Algorithmus ergänzen
        pass
